/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: EOFetchSpecification.h,v 1.2 2003/07/11 17:12:46 jan Exp $

#ifndef __EOFetchSpecification_h__
#define __EOFetchSpecification_h__

#import <Foundation/NSObject.h>

@class NSArray, NSString, NSDictionary;
@class EOQualifier;

@interface EOFetchSpecification : NSObject < NSCopying >
{
  NSString     *entityName;
  EOQualifier  *qualifier;
  NSArray      *sortOrderings;
  BOOL         usesDistinct;
  BOOL         locksObjects;
  unsigned     fetchLimit;
  NSDictionary *hints;
}

+ (EOFetchSpecification *)fetchSpecificationWithEntityName:(NSString *)_ename
  qualifier:(EOQualifier *)_qualifier
  sortOrderings:(NSArray *)sortOrderings;

- (id)initWithEntityName:(NSString *)_name
  qualifier:(EOQualifier *)_qualifier
  sortOrderings:(NSArray *)_sortOrderings
  usesDistinct:(BOOL)_dflag;

/* plist initializers */

- (id)initWithDictionary:(NSDictionary *)_dictionary;
- (id)initWithString:(NSString *)_string;
- (id)initWithPropertyList:(id)_plist;

/* accessors */

- (void)setEntityName:(NSString *)_name;
- (NSString *)entityName;

- (void)setQualifier:(EOQualifier *)_qualifier;
- (EOQualifier *)qualifier;

- (void)setSortOrderings:(NSArray *)_orderings;
- (NSArray *)sortOrderings;

- (void)setUsesDistinct:(BOOL)_flag;
- (BOOL)usesDistinct;

- (void)setLocksObjects:(BOOL)_flag;
- (BOOL)locksObjects;

- (void)setFetchLimit:(unsigned)_limit;
- (unsigned)fetchLimit;

- (void)setHints:(NSDictionary *)_hints;
- (NSDictionary *)hints;

/* bindings */

- (EOFetchSpecification *)fetchSpecificationWithQualifierBindings:(NSDictionary *)_bindings;

/* remapping keys */

- (EOFetchSpecification *)fetchSpecificationByApplyingKeyMap:(NSDictionary *)_m;

@end

#endif /* __EOFetchSpecification_h__ */
